@@ -30,7 +30,8 @@
 extern "C" {
 #endif
 
-#include <compositor.h>
+#include <libweston/libweston.h>
+#include <libweston/config-parser.h>
 
 void
 screenshooter_create(struct weston_compositor *ec);
@@ -45,24 +46,26 @@ struct weston_process {
 	struct wl_list link;
 };
 
-struct wl_client *
+struct custom_env;
+
+bool
 weston_client_launch(struct weston_compositor *compositor,
 		     struct weston_process *proc,
-		     const char *path,
+		     struct custom_env *custom_env,
+		     int *fds_no_cloexec,
+		     size_t num_fds_no_cloexec,
 		     weston_process_cleanup_func_t cleanup);
 
 struct wl_client *
 weston_client_start(struct weston_compositor *compositor, const char *path);
 
 void
-weston_watch_process(struct weston_process *process);
+wet_watch_process(struct weston_compositor *compositor,
+				  struct weston_process *process);
 
 struct weston_config *
 wet_get_config(struct weston_compositor *compositor);
 
-void *
-wet_load_module_entrypoint(const char *name, const char *entrypoint);
-
 int
 wet_shell_init(struct weston_compositor *ec,
 	       int *argc, char *argv[]);
@@ -78,7 +81,10 @@ module_init(struct weston_compositor *compositor,
 	    int *argc, char *argv[]);
 
 char *
-wet_get_binary_path(const char *name);
+wet_get_libexec_path(const char *name);
+
+char *
+wet_get_bindir_path(const char *name);
 
 int
 wet_load_xwayland(struct weston_compositor *comp);
@@ -91,6 +97,15 @@ text_backend_init(struct weston_compositor *ec);
 void
 text_backend_destroy(struct text_backend *text_backend);
 
+/*
+ * Return value from wet_main() when
+ * weston_testsuite_quirks::required_capabilities are not met.
+ */
+#define WET_MAIN_RET_MISSING_CAPS 77
+
+int
+wet_main(int argc, char *argv[], const struct weston_testsuite_data *test_data);
+
 #ifdef  __cplusplus
 }
 #endif
