@@ -37,14 +37,13 @@
 #include "compositor/weston.h"
 #include "fullscreen-shell-unstable-v1-server-protocol.h"
 #include "shared/helpers.h"
+#include <libweston/shell-utils.h>
 
 struct fullscreen_shell {
 	struct wl_client *client;
 	struct wl_listener client_destroyed;
+	struct wl_listener destroy_listener;
 	struct weston_compositor *compositor;
-	/* XXX: missing compositor destroy listener
-	 * https://gitlab.freedesktop.org/wayland/weston/issues/299
-	 */
 
 	struct weston_layer layer;
 	struct wl_list output_list;
@@ -80,7 +79,7 @@ struct fs_output {
 	struct weston_surface *surface;
 	struct wl_listener surface_destroyed;
 	struct weston_view *view;
-	struct weston_view *black_view;
+	struct weston_curtain *curtain;
 	struct weston_transform transform; /* matrix from x, y */
 
 	int presented_for_mode;
@@ -222,41 +221,32 @@ seat_created(struct wl_listener *l, void *data)
 }
 
 static void
-black_surface_committed(struct weston_surface *es, int32_t sx, int32_t sy)
+black_surface_committed(struct weston_surface *es,
+			struct weston_coord_surface new_origin)
 {
 }
 
-static struct weston_view *
-create_black_surface(struct weston_compositor *ec, struct fs_output *fsout,
-		     float x, float y, int w, int h)
+static struct weston_curtain *
+create_curtain(struct weston_compositor *ec, struct fs_output *fsout,
+	       float x, float y, int w, int h)
 {
-	struct weston_surface *surface = NULL;
-	struct weston_view *view;
+	struct weston_curtain_params curtain_params = {
+		.r = 0.0, .g = 0.0, .b = 0.0, .a = 1.0,
+		.x = x, .y = y, .width = w, .height = h,
+		.surface_committed = black_surface_committed,
+		.get_label = NULL,
+		.surface_private = fsout,
+		.capture_input = true,
+	};
+	struct weston_curtain *curtain;
 
-	surface = weston_surface_create(ec);
-	if (surface == NULL) {
-		weston_log("no memory\n");
-		return NULL;
-	}
-	view = weston_view_create(surface);
-	if (!view) {
-		weston_surface_destroy(surface);
+	curtain = weston_shell_utils_curtain_create(ec, &curtain_params);
+	if (!curtain) {
 		weston_log("no memory\n");
 		return NULL;
 	}
 
-	surface->committed = black_surface_committed;
-	surface->committed_private = fsout;
-	weston_surface_set_color(surface, 0.0f, 0.0f, 0.0f, 1.0f);
-	pixman_region32_fini(&surface->opaque);
-	pixman_region32_init_rect(&surface->opaque, 0, 0, w, h);
-	pixman_region32_fini(&surface->input);
-	pixman_region32_init_rect(&surface->input, 0, 0, w, h);
-
-	weston_surface_set_size(surface, w, h);
-	weston_view_set_position(view, x, y);
-
-	return view;
+	return curtain;
 }
 
 static void
@@ -274,6 +264,7 @@ fs_output_destroy(struct fs_output *fsout)
 	fs_output_set_surface(fsout, NULL, 0, 0, 0);
 	fs_output_clear_pending(fsout);
 
+	weston_shell_utils_curtain_destroy(fsout->curtain);
 	wl_list_remove(&fsout->link);
 
 	if (fsout->output)
@@ -311,8 +302,7 @@ pending_surface_destroyed(struct wl_listener *listener, void *data)
 }
 
 static void
-configure_presented_surface(struct weston_surface *surface, int32_t sx,
-			    int32_t sy);
+configure_presented_surface_internal(struct weston_surface *surface);
 
 static struct fs_output *
 fs_output_create(struct fullscreen_shell *shell, struct weston_output *output)
@@ -333,13 +323,12 @@ fs_output_create(struct fullscreen_shell *shell, struct weston_output *output)
 
 	fsout->surface_destroyed.notify = surface_destroyed;
 	fsout->pending.surface_destroyed.notify = pending_surface_destroyed;
-	fsout->black_view = create_black_surface(shell->compositor, fsout,
-						 output->x, output->y,
-						 output->width, output->height);
-	fsout->black_view->surface->is_mapped = true;
-	fsout->black_view->is_mapped = true;
+	fsout->curtain = create_curtain(shell->compositor, fsout,
+					output->x, output->y,
+					output->width, output->height);
+	fsout->curtain->view->is_mapped = true;
 	weston_layer_entry_insert(&shell->layer.view_list,
-		       &fsout->black_view->layer_link);
+			          &fsout->curtain->view->layer_link);
 	wl_list_init(&fsout->transform.link);
 
 	if (!wl_list_empty(&shell->default_surface_list)) {
@@ -347,7 +336,7 @@ fs_output_create(struct fullscreen_shell *shell, struct weston_output *output)
 				    struct fs_client_surface, link);
 
 		fs_output_set_surface(fsout, surf->surface, surf->method, 0, 0);
-		configure_presented_surface(surf->surface, 0, 0);
+		configure_presented_surface_internal(surf->surface);
 	}
 
 	return fsout;
@@ -373,57 +362,6 @@ restore_output_mode(struct weston_output *output)
 		weston_output_mode_switch_to_native(output);
 }
 
-/*
- * Returns the bounding box of a surface and all its sub-surfaces,
- * in surface-local coordinates. */
-static void
-surface_subsurfaces_boundingbox(struct weston_surface *surface, int32_t *x,
-				int32_t *y, int32_t *w, int32_t *h) {
-	pixman_region32_t region;
-	pixman_box32_t *box;
-	struct weston_subsurface *subsurface;
-
-	pixman_region32_init_rect(&region, 0, 0,
-	                          surface->width,
-	                          surface->height);
-
-	wl_list_for_each(subsurface, &surface->subsurface_list, parent_link) {
-		pixman_region32_union_rect(&region, &region,
-		                           subsurface->position.x,
-		                           subsurface->position.y,
-		                           subsurface->surface->width,
-		                           subsurface->surface->height);
-	}
-
-	box = pixman_region32_extents(&region);
-	if (x)
-		*x = box->x1;
-	if (y)
-		*y = box->y1;
-	if (w)
-		*w = box->x2 - box->x1;
-	if (h)
-		*h = box->y2 - box->y1;
-
-	pixman_region32_fini(&region);
-}
-
-static void
-fs_output_center_view(struct fs_output *fsout)
-{
-	int32_t surf_x, surf_y, surf_width, surf_height;
-	float x, y;
-	struct weston_output *output = fsout->output;
-
-	surface_subsurfaces_boundingbox(fsout->view->surface, &surf_x, &surf_y,
-					&surf_width, &surf_height);
-
-	x = output->x + (output->width - surf_width) / 2 - surf_x / 2;
-	y = output->y + (output->height - surf_height) / 2 - surf_y / 2;
-
-	weston_view_set_position(fsout->view, x, y);
-}
-
 static void
 fs_output_scale_view(struct fs_output *fsout, float width, float height)
 {
@@ -433,8 +371,8 @@ fs_output_scale_view(struct fs_output *fsout, float width, float height)
 	struct weston_view *view = fsout->view;
 	struct weston_output *output = fsout->output;
 
-	surface_subsurfaces_boundingbox(view->surface, &surf_x, &surf_y,
-					&surf_width, &surf_height);
+	weston_shell_utils_subsurfaces_boundingbox(view->surface, &surf_x, &surf_y,
+						   &surf_width, &surf_height);
 
 	if (output->width == surf_width && output->height == surf_height) {
 		weston_view_set_position(view,
@@ -478,9 +416,9 @@ fs_output_configure_simple(struct fs_output *fsout,
 	wl_list_remove(&fsout->transform.link);
 	wl_list_init(&fsout->transform.link);
 
-	surface_subsurfaces_boundingbox(fsout->view->surface,
-					&surf_x, &surf_y,
-					&surf_width, &surf_height);
+	weston_shell_utils_subsurfaces_boundingbox(fsout->view->surface,
+						   &surf_x, &surf_y,
+						   &surf_width, &surf_height);
 
 	output_aspect = (float) output->width / (float) output->height;
 	surface_aspect = (float) surf_width / (float) surf_height;
@@ -488,7 +426,7 @@ fs_output_configure_simple(struct fs_output *fsout,
 	switch (fsout->method) {
 	case ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_DEFAULT:
 	case ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_CENTER:
-		fs_output_center_view(fsout);
+		weston_shell_utils_center_on_output(fsout->view, fsout->output);
 		break;
 
 	case ZWP_FULLSCREEN_SHELL_V1_PRESENT_METHOD_ZOOM:
@@ -520,10 +458,10 @@ fs_output_configure_simple(struct fs_output *fsout,
 		break;
 	}
 
-	weston_view_set_position(fsout->black_view,
+	weston_view_set_position(fsout->curtain->view,
 				 fsout->output->x - surf_x,
 				 fsout->output->y - surf_y);
-	weston_surface_set_size(fsout->black_view->surface,
+	weston_surface_set_size(fsout->curtain->view->surface,
 				fsout->output->width,
 				fsout->output->height);
 }
@@ -539,14 +477,14 @@ fs_output_configure_for_mode(struct fs_output *fsout,
 	if (fsout->pending.surface != configured_surface) {
 		/* Nothing to really reconfigure.  We'll just recenter the
 		 * view in case they played with subsurfaces */
-		fs_output_center_view(fsout);
+		weston_shell_utils_center_on_output(fsout->view, fsout->output);
 		return;
 	}
 
 	/* We have a pending surface */
-	surface_subsurfaces_boundingbox(fsout->pending.surface,
-					&surf_x, &surf_y,
-					&surf_width, &surf_height);
+	weston_shell_utils_subsurfaces_boundingbox(fsout->pending.surface,
+						   &surf_x, &surf_y,
+						   &surf_width, &surf_height);
 
 	/* The actual output mode is in physical units.  We need to
 	 * transform the surface size to physical unit size by flipping and
@@ -623,8 +561,14 @@ fs_output_configure(struct fs_output *fsout,
 }
 
 static void
-configure_presented_surface(struct weston_surface *surface, int32_t sx,
-			    int32_t sy)
+configure_presented_surface(struct weston_surface *surface,
+			    struct weston_coord_surface new_origin)
+{
+	configure_presented_surface_internal(surface);
+}
+
+static void
+configure_presented_surface_internal(struct weston_surface *surface)
 {
 	struct fullscreen_shell *shell = surface->committed_private;
 	struct fs_output *fsout;
@@ -670,11 +614,14 @@ fs_output_apply_pending(struct fs_output *fsout)
 			return;
 		}
 		fsout->view->is_mapped = true;
+		fsout->surface->is_mapped = true;
 
 		wl_signal_add(&fsout->surface->destroy_signal,
 			      &fsout->surface_destroyed);
 		weston_layer_entry_insert(&fsout->shell->layer.view_list,
 			       &fsout->view->layer_link);
+
+		weston_view_geometry_dirty(fsout->view);
 	}
 
 	fs_output_clear_pending(fsout);
@@ -899,6 +846,28 @@ bind_fullscreen_shell(struct wl_client *client, void *data, uint32_t version,
 			ZWP_FULLSCREEN_SHELL_V1_CAPABILITY_ARBITRARY_MODES);
 }
 
+static void
+fullscreen_shell_destroy(struct wl_listener *listener, void *data)
+{
+	struct fs_output *fs_output, *fs_output_next;
+	struct fs_client_surface *surf;
+	struct fullscreen_shell *shell =
+		container_of(listener, struct fullscreen_shell, destroy_listener);
+
+	/* remove the curtain(s) */
+	wl_list_for_each_safe(fs_output, fs_output_next, &shell->output_list, link)
+		fs_output_destroy(fs_output);
+
+	if (!wl_list_empty(&shell->default_surface_list)) {
+		surf = container_of(shell->default_surface_list.prev,
+				struct fs_client_surface, link);
+		remove_default_surface(surf);
+	}
+
+	weston_layer_fini(&shell->layer);
+	free(shell);
+}
+
 WL_EXPORT int
 wet_shell_init(struct weston_compositor *compositor,
 	       int *argc, char *argv[])
@@ -911,7 +880,16 @@ wet_shell_init(struct weston_compositor *compositor,
 	if (shell == NULL)
 		return -1;
 
+
 	shell->compositor = compositor;
+
+	if (!weston_compositor_add_destroy_listener_once(compositor,
+							 &shell->destroy_listener,
+							 fullscreen_shell_destroy)) {
+		free(shell);
+		return 0;
+	}
+
 	wl_list_init(&shell->default_surface_list);
 
 	shell->client_destroyed.notify = client_destroyed;
@@ -937,5 +915,6 @@ wet_shell_init(struct weston_compositor *compositor,
 			 &zwp_fullscreen_shell_v1_interface, 1, shell,
 			 bind_fullscreen_shell);
 
+	screenshooter_create(compositor);
 	return 0;
 }
