@@ -30,7 +30,7 @@
 
 #include <libweston/libweston.h>
 #include "ivi-layout-export.h"
-#include <libweston-desktop/libweston-desktop.h>
+#include <libweston/desktop.h>
 
 struct ivi_layout_view {
 	struct wl_list link;	/* ivi_layout::view_list */
@@ -55,6 +55,8 @@ struct ivi_layout_surface {
 	struct weston_surface *surface;
 	struct weston_desktop_surface *weston_desktop_surface;
 
+	struct ivi_layout_view *ivi_view;
+
 	struct ivi_layout_surface_properties prop;
 
 	struct {
@@ -90,13 +92,17 @@ struct ivi_layout_layer {
 };
 
 struct ivi_layout {
-	struct weston_compositor *compositor;
+	struct ivi_shell *shell;
 
 	struct wl_list surface_list;	/* ivi_layout_surface::link */
 	struct wl_list layer_list;	/* ivi_layout_layer::link */
 	struct wl_list screen_list;	/* ivi_layout_screen::link */
 	struct wl_list view_list;	/* ivi_layout_view::link */
 
+	struct {
+		struct wl_signal destroy_signal;
+	} shell_notification;
+
 	struct {
 		struct wl_signal created;
 		struct wl_signal removed;
@@ -109,10 +115,25 @@ struct ivi_layout {
 		struct wl_signal configure_desktop_changed;
 	} surface_notification;
 
+	struct {
+		struct wl_signal configure_changed;
+		struct wl_signal show;
+		struct wl_signal hide;
+		struct wl_signal update;
+	} input_panel_notification;
+
 	struct weston_layer layout_layer;
 
 	struct ivi_layout_transition_set *transitions;
 	struct wl_list pending_transition_list;	/* transition_node::link */
+
+	struct wl_listener output_created;
+	struct wl_listener output_destroyed;
+
+	struct {
+		struct ivi_layout_surface *ivisurf;
+		pixman_box32_t cursor_rectangle;
+	} text_input;
 };
 
 struct ivi_layout *get_instance(void);
@@ -170,14 +191,14 @@ int32_t
 ivi_layout_commit_changes(void);
 uint32_t
 ivi_layout_get_id_of_surface(struct ivi_layout_surface *ivisurf);
-int32_t
+void
 ivi_layout_surface_set_destination_rectangle(struct ivi_layout_surface *ivisurf,
 					     int32_t x, int32_t y,
 					     int32_t width, int32_t height);
 int32_t
 ivi_layout_surface_set_opacity(struct ivi_layout_surface *ivisurf,
 			       wl_fixed_t opacity);
-int32_t
+void
 ivi_layout_surface_set_visibility(struct ivi_layout_surface *ivisurf,
 				  bool newVisibility);
 void
@@ -188,14 +209,14 @@ ivi_layout_get_surface_from_id(uint32_t id_surface);
 int32_t
 ivi_layout_layer_set_opacity(struct ivi_layout_layer *ivilayer,
 			     wl_fixed_t opacity);
-int32_t
+void
 ivi_layout_layer_set_visibility(struct ivi_layout_layer *ivilayer,
 				bool newVisibility);
-int32_t
+void
 ivi_layout_layer_set_destination_rectangle(struct ivi_layout_layer *ivilayer,
 					   int32_t x, int32_t y,
 					   int32_t width, int32_t height);
-int32_t
+void
 ivi_layout_layer_set_render_order(struct ivi_layout_layer *ivilayer,
 				  struct ivi_layout_surface **pSurface,
 				  int32_t number);
