@@ -33,7 +33,7 @@
 #include "kiosk-shell-grab.h"
 #include "compositor/weston.h"
 #include "shared/helpers.h"
-#include "shared/shell-utils.h"
+#include <libweston/shell-utils.h>
 
 #include <libweston/xwayland-api.h>
 
@@ -90,7 +90,6 @@ transform_handler(struct wl_listener *listener, void *data)
 	struct weston_surface *surface = data;
 	struct kiosk_shell_surface *shsurf = get_kiosk_shell_surface(surface);
 	const struct weston_xwayland_surface_api *api;
-	int x, y;
 
 	if (!shsurf)
 		return;
@@ -107,10 +106,9 @@ transform_handler(struct wl_listener *listener, void *data)
 	if (!weston_view_is_mapped(shsurf->view))
 		return;
 
-	x = shsurf->view->geometry.x;
-	y = shsurf->view->geometry.y;
-
-	api->send_position(surface, x, y);
+	api->send_position(surface,
+			   shsurf->view->geometry.pos_offset.x,
+			   shsurf->view->geometry.pos_offset.y);
 }
 
 /*
@@ -185,11 +183,11 @@ kiosk_shell_surface_find_best_output(struct kiosk_shell_surface *shsurf)
 	if (root->output)
 		return root->output;
 
-	output = get_focused_output(shsurf->shell->compositor);
+	output = weston_shell_utils_get_focused_output(shsurf->shell->compositor);
 	if (output)
 		return output;
 
-	output = get_default_output(shsurf->shell->compositor);
+	output = weston_shell_utils_get_default_output(shsurf->shell->compositor);
 	if (output)
 		return output;
 
@@ -299,7 +297,7 @@ kiosk_shell_surface_reconfigure_for_output(struct kiosk_shell_surface *shsurf)
 						shsurf->output->height);
 	}
 
-	center_on_output(shsurf->view, shsurf->output);
+	weston_shell_utils_center_on_output(shsurf->view, shsurf->output);
 	weston_view_update_transform(shsurf->view);
 }
 
@@ -413,6 +411,7 @@ kiosk_shell_surface_activate(struct kiosk_shell_surface *shsurf,
 	weston_layer_entry_insert(&shsurf->shell->normal_layer.view_list,
 				  &shsurf->view->layer_link);
 	weston_view_geometry_dirty(shsurf->view);
+	weston_view_update_transform(shsurf->view);
 	weston_surface_damage(shsurf->view->surface);
 }
 
@@ -480,13 +479,14 @@ static void
 kiosk_shell_output_recreate_background(struct kiosk_shell_output *shoutput)
 {
 	struct kiosk_shell *shell = shoutput->shell;
+	struct weston_compositor *ec = shell->compositor;
 	struct weston_output *output = shoutput->output;
 	struct weston_config_section *shell_section = NULL;
 	uint32_t bg_color = 0x0;
-	struct weston_solid_color_surface solid_surface = {};
+	struct weston_curtain_params curtain_params = {};
 
-	if (shoutput->background_view)
-		weston_surface_destroy(shoutput->background_view->surface);
+	if (shoutput->curtain)
+		weston_shell_utils_curtain_destroy(shoutput->curtain);
 
 	if (!output)
 		return;
@@ -497,31 +497,33 @@ kiosk_shell_output_recreate_background(struct kiosk_shell_output *shoutput)
 		weston_config_section_get_color(shell_section, "background-color",
 						&bg_color, 0x00000000);
 
-	solid_surface.r = ((bg_color >> 16) & 0xff) / 255.0;
-	solid_surface.g = ((bg_color >> 8) & 0xff) / 255.0;
-	solid_surface.b = ((bg_color >> 0) & 0xff) / 255.0;
+	curtain_params.r = ((bg_color >> 16) & 0xff) / 255.0;
+	curtain_params.g = ((bg_color >> 8) & 0xff) / 255.0;
+	curtain_params.b = ((bg_color >> 0) & 0xff) / 255.0;
+	curtain_params.a = 1.0;
 
-	solid_surface.get_label = kiosk_shell_background_surface_get_label;
-	solid_surface.surface_committed = NULL;
-	solid_surface.surface_private = NULL;
+	curtain_params.x = output->x;
+	curtain_params.y = output->y;
+	curtain_params.width = output->width;
+	curtain_params.height = output->height;
 
-	shoutput->background_view =
-			create_solid_color_surface(shoutput->shell->compositor,
-						   &solid_surface,
-						   output->x, output->y,
-						   output->width,
-						   output->height);
+	curtain_params.capture_input = true;
 
-	weston_surface_set_role(shoutput->background_view->surface,
+	curtain_params.get_label = kiosk_shell_background_surface_get_label;
+	curtain_params.surface_committed = NULL;
+	curtain_params.surface_private = NULL;
+
+	shoutput->curtain = weston_shell_utils_curtain_create(ec, &curtain_params);
+
+	weston_surface_set_role(shoutput->curtain->view->surface,
 				"kiosk-shell-background", NULL, 0);
 
 	weston_layer_entry_insert(&shell->background_layer.view_list,
-				  &shoutput->background_view->layer_link);
+				  &shoutput->curtain->view->layer_link);
 
-	shoutput->background_view->is_mapped = true;
-	shoutput->background_view->surface->is_mapped = true;
-	shoutput->background_view->surface->output = output;
-	weston_view_set_output(shoutput->background_view, output);
+	shoutput->curtain->view->is_mapped = true;
+	shoutput->curtain->view->surface->output = output;
+	weston_view_set_output(shoutput->curtain->view, output);
 }
 
 static void
@@ -530,8 +532,8 @@ kiosk_shell_output_destroy(struct kiosk_shell_output *shoutput)
 	shoutput->output = NULL;
 	shoutput->output_destroy_listener.notify = NULL;
 
-	if (shoutput->background_view)
-		weston_surface_destroy(shoutput->background_view->surface);
+	if (shoutput->curtain)
+		weston_shell_utils_curtain_destroy(shoutput->curtain);
 
 	wl_list_remove(&shoutput->output_destroy_listener.link);
 	wl_list_remove(&shoutput->link);
@@ -632,7 +634,7 @@ desktop_surface_added(struct weston_desktop_surface *desktop_surface,
 	if (!shsurf)
 		return;
 
-	weston_surface_set_label_func(surface, surface_get_label);
+	weston_surface_set_label_func(surface, weston_shell_utils_surface_get_label);
 	kiosk_shell_surface_set_fullscreen(shsurf, NULL);
 }
 
@@ -776,7 +778,8 @@ desktop_surface_committed(struct weston_desktop_surface *desktop_surface,
 
 	if (!weston_surface_is_mapped(surface) || (is_resized && is_fullscreen)) {
 		if (is_fullscreen || !shsurf->xwayland.is_set) {
-			center_on_output(shsurf->view, shsurf->output);
+			weston_shell_utils_center_on_output(shsurf->view,
+							    shsurf->output);
 		} else {
 			struct weston_geometry geometry =
 				weston_desktop_surface_get_geometry(desktop_surface);
@@ -795,7 +798,7 @@ desktop_surface_committed(struct weston_desktop_surface *desktop_surface,
 		struct kiosk_shell_seat *kiosk_seat;
 
 		shsurf->view->is_mapped = true;
-		surface->is_mapped = true;
+		weston_surface_map(surface);
 
 		kiosk_seat = get_kiosk_shell_seat(seat);
 		if (seat && kiosk_seat)
@@ -804,16 +807,22 @@ desktop_surface_committed(struct weston_desktop_surface *desktop_surface,
 	}
 
 	if (!is_fullscreen && (sx != 0 || sy != 0)) {
-		float from_x, from_y;
-		float to_x, to_y;
-		float x, y;
+		struct weston_coord_surface from_s, to_s;
+		struct weston_coord_global from_g, to_g;
+		struct weston_coord_global offset, pos;
 
-		weston_view_to_global_float(shsurf->view, 0, 0, &from_x, &from_y);
-		weston_view_to_global_float(shsurf->view, sx, sy, &to_x, &to_y);
-		x = shsurf->view->geometry.x + to_x - from_x;
-		y = shsurf->view->geometry.y + to_y - from_y;
+		from_s = weston_coord_surface(0, 0,
+					      shsurf->view->surface);
+		to_s = weston_coord_surface(sx, sy,
+					    shsurf->view->surface);
 
-		weston_view_set_position(shsurf->view, x, y);
+		from_g = weston_coord_surface_to_global(shsurf->view, from_s);
+		to_g = weston_coord_surface_to_global(shsurf->view, to_s);
+		offset.c = weston_coord_sub(to_g.c, from_g.c);
+		pos.c = weston_coord_add(shsurf->view->geometry.pos_offset,
+					 offset.c);
+
+		weston_view_set_position(shsurf->view, pos.c.x, pos.c.y);
 		weston_view_update_transform(shsurf->view);
 	}
 
@@ -948,6 +957,17 @@ desktop_surface_set_xwayland_position(struct weston_desktop_surface *desktop_sur
 	shsurf->xwayland.is_set = true;
 }
 
+static void
+desktop_surface_get_position(struct weston_desktop_surface *desktop_surface,
+			     int32_t *x, int32_t *y, void *shell)
+{
+	struct kiosk_shell_surface *shsurf =
+		weston_desktop_surface_get_user_data(desktop_surface);
+
+	*x = shsurf->view->geometry.pos_offset.x;
+	*y = shsurf->view->geometry.pos_offset.y;
+}
+
 static const struct weston_desktop_api kiosk_shell_desktop_api = {
 	.struct_size = sizeof(struct weston_desktop_api),
 	.surface_added = desktop_surface_added,
@@ -962,6 +982,7 @@ static const struct weston_desktop_api kiosk_shell_desktop_api = {
 	.ping_timeout = desktop_surface_ping_timeout,
 	.pong = desktop_surface_pong,
 	.set_xwayland_position = desktop_surface_set_xwayland_position,
+	.get_position = desktop_surface_get_position,
 };
 
 /*
@@ -1049,6 +1070,10 @@ kiosk_shell_touch_to_activate_binding(struct weston_touch *touch,
 static void
 kiosk_shell_add_bindings(struct kiosk_shell *shell)
 {
+	uint32_t mod = 0;
+
+	mod = weston_config_get_binding_modifier(shell->config, MODIFIER_SUPER);
+
 	weston_compositor_add_button_binding(shell->compositor, BTN_LEFT, 0,
 					     kiosk_shell_click_to_activate_binding,
 					     shell);
@@ -1058,6 +1083,8 @@ kiosk_shell_add_bindings(struct kiosk_shell *shell)
 	weston_compositor_add_touch_binding(shell->compositor, 0,
 					    kiosk_shell_touch_to_activate_binding,
 					    shell);
+
+	weston_install_debug_key_binding(shell->compositor, mod);
 }
 
 static void
@@ -1107,8 +1134,8 @@ kiosk_shell_handle_output_moved(struct wl_listener *listener, void *data)
 		if (view->output != output)
 			continue;
 		weston_view_set_position(view,
-					 view->geometry.x + output->move_x,
-					 view->geometry.y + output->move_y);
+					 view->geometry.pos_offset.x + output->move_x,
+					 view->geometry.pos_offset.y + output->move_y);
 	}
 
 	wl_list_for_each(view, &shell->normal_layer.view_list.link,
@@ -1116,8 +1143,8 @@ kiosk_shell_handle_output_moved(struct wl_listener *listener, void *data)
 		if (view->output != output)
 			continue;
 		weston_view_set_position(view,
-					 view->geometry.x + output->move_x,
-					 view->geometry.y + output->move_y);
+					 view->geometry.pos_offset.x + output->move_x,
+					 view->geometry.pos_offset.y + output->move_y);
 	}
 }
 
