@@ -39,7 +39,6 @@
 #include <sys/shm.h>
 #include <linux/input.h>
 
-#include <drm_fourcc.h>
 #include <xcb/xcb.h>
 #include <xcb/shm.h>
 #ifdef HAVE_XCB_XKB
@@ -58,11 +57,14 @@
 #include "shared/timespec-util.h"
 #include "shared/file-util.h"
 #include "renderer-gl/gl-renderer.h"
+#include "shared/weston-drm-fourcc.h"
 #include "shared/weston-egl-ext.h"
+#include "shared/xalloc.h"
 #include "pixman-renderer.h"
 #include "presentation-time-server-protocol.h"
 #include "linux-dmabuf.h"
 #include "linux-explicit-synchronization.h"
+#include <libweston/pixel-formats.h>
 #include <libweston/windowed-output-api.h>
 
 #define DEFAULT_AXIS_STEP_DISTANCE 10
@@ -92,15 +94,13 @@ struct x11_backend {
 	uint8_t			 xkb_event_base;
 	int			 fullscreen;
 	int			 no_input;
-	int			 use_pixman;
 
 	int			 has_net_wm_state_fullscreen;
 
 	/* We could map multi-pointer X to multiple wayland seats, but
 	 * for now we only support core X input. */
 	struct weston_seat		 core_seat;
-	double				 prev_x;
-	double				 prev_y;
+	struct weston_coord_global	 prev_pos;
 
 	struct {
 		xcb_atom_t		 wm_protocols;
@@ -119,6 +119,10 @@ struct x11_backend {
 		xcb_atom_t		 cardinal;
 		xcb_atom_t		 xkb_names;
 	} atom;
+	xcb_generic_event_t *prev_event;
+
+	const struct pixel_format_info **formats;
+	unsigned int formats_count;
 };
 
 struct x11_head {
@@ -127,6 +131,7 @@ struct x11_head {
 
 struct x11_output {
 	struct weston_output	base;
+	struct x11_backend	*backend;
 
 	xcb_window_t		window;
 	struct weston_mode	mode;
@@ -135,7 +140,7 @@ struct x11_output {
 
 	xcb_gc_t		gc;
 	xcb_shm_seg_t		segment;
-	pixman_image_t	       *hw_surface;
+	struct weston_renderbuffer *renderbuffer;
 	int			shm_id;
 	void		       *buf;
 	uint8_t			depth;
@@ -149,24 +154,32 @@ struct window_delete_data {
 	xcb_window_t		window;
 };
 
-struct gl_renderer_interface *gl_renderer;
+static void
+x11_destroy(struct weston_backend *backend);
 
 static inline struct x11_head *
 to_x11_head(struct weston_head *base)
 {
+	if (base->backend->destroy != x11_destroy)
+		return NULL;
 	return container_of(base, struct x11_head, base);
 }
 
+static void
+x11_output_destroy(struct weston_output *base);
+
 static inline struct x11_output *
 to_x11_output(struct weston_output *base)
 {
+	if (base->destroy != x11_output_destroy)
+		return NULL;
 	return container_of(base, struct x11_output, base);
 }
 
 static inline struct x11_backend *
-to_x11_backend(struct weston_compositor *base)
+to_x11_backend(struct weston_backend *backend)
 {
-	return container_of(base->backend, struct x11_backend, base);
+	return container_of(backend, struct x11_backend, base);
 }
 
 static xcb_screen_t *
@@ -417,13 +430,16 @@ x11_output_start_repaint_loop(struct weston_output *output)
 
 static int
 x11_output_repaint_gl(struct weston_output *output_base,
-		      pixman_region32_t *damage,
-		      void *repaint_data)
+		      pixman_region32_t *damage)
 {
 	struct x11_output *output = to_x11_output(output_base);
-	struct weston_compositor *ec = output->base.compositor;
+	struct weston_compositor *ec;
 
-	ec->renderer->repaint_output(output_base, damage);
+	assert(output);
+
+	ec = output->base.compositor;
+
+	ec->renderer->repaint_output(output_base, damage, NULL);
 
 	pixman_region32_subtract(&ec->primary_plane.damage,
 				 &ec->primary_plane.damage, damage);
@@ -436,8 +452,7 @@ static void
 set_clip_for_output(struct weston_output *output_base, pixman_region32_t *region)
 {
 	struct x11_output *output = to_x11_output(output_base);
-	struct weston_compositor *ec = output->base.compositor;
-	struct x11_backend *b = to_x11_backend(ec);
+	struct x11_backend *b;
 	pixman_region32_t transformed_region;
 	pixman_box32_t *rects;
 	xcb_rectangle_t *output_rects;
@@ -445,14 +460,15 @@ set_clip_for_output(struct weston_output *output_base, pixman_region32_t *region
 	int nrects, i;
 	xcb_generic_error_t *err;
 
+	if (!output)
+		return;
+
+	b = output->backend;
+
 	pixman_region32_init(&transformed_region);
-	pixman_region32_copy(&transformed_region, region);
-	pixman_region32_translate(&transformed_region,
-				  -output_base->x, -output_base->y);
-	weston_transformed_region(output_base->width, output_base->height,
-				  output_base->transform,
-				  output_base->current_scale,
-				  &transformed_region, &transformed_region);
+	weston_region_global_to_output(&transformed_region,
+				       output_base,
+				       region);
 
 	rects = pixman_region32_rectangles(&transformed_region, &nrects);
 	output_rects = calloc(nrects, sizeof(xcb_rectangle_t));
@@ -486,27 +502,35 @@ set_clip_for_output(struct weston_output *output_base, pixman_region32_t *region
 
 static int
 x11_output_repaint_shm(struct weston_output *output_base,
-		       pixman_region32_t *damage,
-		       void *repaint_data)
+		       pixman_region32_t *damage)
 {
 	struct x11_output *output = to_x11_output(output_base);
-	struct weston_compositor *ec = output->base.compositor;
-	struct x11_backend *b = to_x11_backend(ec);
+	const struct weston_renderer *renderer;
+	pixman_image_t *image;
+	struct weston_compositor *ec;
+	struct x11_backend *b;
 	xcb_void_cookie_t cookie;
 	xcb_generic_error_t *err;
 
-	pixman_renderer_output_set_buffer(output_base, output->hw_surface);
-	ec->renderer->repaint_output(output_base, damage);
+	assert(output);
+
+	ec = output->base.compositor;
+	renderer = ec->renderer;
+	b = output->backend;
+
+	image = renderer->pixman->renderbuffer_get_image(output->renderbuffer);
+
+	ec->renderer->repaint_output(output_base, damage, output->renderbuffer);
 
 	pixman_region32_subtract(&ec->primary_plane.damage,
 				 &ec->primary_plane.damage, damage);
 	set_clip_for_output(output_base, damage);
 	cookie = xcb_shm_put_image_checked(b->conn, output->window, output->gc,
-					pixman_image_get_width(output->hw_surface),
-					pixman_image_get_height(output->hw_surface),
+					pixman_image_get_width(image),
+					pixman_image_get_height(image),
 					0, 0,
-					pixman_image_get_width(output->hw_surface),
-					pixman_image_get_height(output->hw_surface),
+					pixman_image_get_width(image),
+					pixman_image_get_height(image),
 					0, 0, output->depth, XCB_IMAGE_FORMAT_Z_PIXMAP,
 					0, output->segment, 0);
 	err = xcb_request_check(b->conn, cookie);
@@ -538,8 +562,8 @@ x11_output_deinit_shm(struct x11_backend *b, struct x11_output *output)
 	xcb_generic_error_t *err;
 	xcb_free_gc(b->conn, output->gc);
 
-	pixman_image_unref(output->hw_surface);
-	output->hw_surface = NULL;
+	weston_renderbuffer_unref(output->renderbuffer);
+	output->renderbuffer = NULL;
 	cookie = xcb_shm_detach_checked(b->conn, output->segment);
 	err = xcb_request_check(b->conn, cookie);
 	if (err) {
@@ -567,13 +591,13 @@ x11_output_set_wm_protocols(struct x11_backend *b,
 }
 
 struct wm_normal_hints {
-    	uint32_t flags;
+	uint32_t flags;
 	uint32_t pad[4];
 	int32_t min_width, min_height;
 	int32_t max_width, max_height;
-    	int32_t width_inc, height_inc;
-    	int32_t min_aspect_x, min_aspect_y;
-    	int32_t max_aspect_x, max_aspect_y;
+	int32_t width_inc, height_inc;
+	int32_t min_aspect_x, min_aspect_y;
+	int32_t max_aspect_x, max_aspect_y;
 	int32_t base_width, base_height;
 	int32_t win_gravity;
 };
@@ -699,18 +723,15 @@ get_depth_of_visual(xcb_screen_t *screen,
 	return 0;
 }
 
-static int
-x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
-	int width, int height)
+static const struct pixel_format_info *
+x11_output_get_shm_pixel_format(struct x11_output *output)
 {
+	struct x11_backend *b = output->backend;
 	xcb_visualtype_t *visual_type;
 	xcb_screen_t *screen;
 	xcb_format_iterator_t fmt;
-	xcb_void_cookie_t cookie;
-	xcb_generic_error_t *err;
 	const xcb_query_extension_reply_t *ext;
 	int bitsperpixel = 0;
-	pixman_format_code_t pixman_format;
 
 	/* Check if SHM is available */
 	ext = xcb_get_extension_data(b->conn, &xcb_shm_id);
@@ -718,7 +739,7 @@ x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
 		/* SHM is missing */
 		weston_log("SHM extension is not available\n");
 		errno = ENOENT;
-		return -1;
+		return NULL;
 	}
 
 	screen = x11_compositor_get_default_screen(b);
@@ -726,7 +747,7 @@ x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
 	if (!visual_type) {
 		weston_log("Failed to lookup visual for root window\n");
 		errno = ENOENT;
-		return -1;
+		return NULL;
 	}
 	weston_log("Found visual, bits per value: %d, red_mask: %.8x, green_mask: %.8x, blue_mask: %.8x\n",
 		visual_type->bits_per_rgb_value,
@@ -752,19 +773,28 @@ x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
 	     visual_type->green_mask == 0x00ff00 &&
 	     visual_type->blue_mask == 0x0000ff) {
 		weston_log("Will use x8r8g8b8 format for SHM surfaces\n");
-		pixman_format = PIXMAN_x8r8g8b8;
+		return pixel_format_get_info_by_pixman(PIXMAN_x8r8g8b8);
 	} else if (bitsperpixel == 16 &&
 	           visual_type->red_mask == 0x00f800 &&
 	           visual_type->green_mask == 0x0007e0 &&
 	           visual_type->blue_mask == 0x00001f) {
 		weston_log("Will use r5g6b5 format for SHM surfaces\n");
-		pixman_format = PIXMAN_r5g6b5;
+		return pixel_format_get_info_by_pixman(PIXMAN_r5g6b5);
 	} else {
 		weston_log("Can't find appropriate format for SHM pixmap\n");
 		errno = ENOTSUP;
-		return -1;
+		return NULL;
 	}
+}
 
+static int
+x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
+		    const struct pixel_format_info *pfmt, int width, int height)
+{
+	struct weston_renderer *renderer = output->base.compositor->renderer;
+	int bitsperpixel = pfmt->bpp;
+	xcb_void_cookie_t cookie;
+	xcb_generic_error_t *err;
 
 	/* Create SHM segment and attach it */
 	output->shm_id = shmget(IPC_PRIVATE, width * height * (bitsperpixel / 8), IPC_CREAT | S_IRWXU);
@@ -790,8 +820,11 @@ x11_output_init_shm(struct x11_backend *b, struct x11_output *output,
 	shmctl(output->shm_id, IPC_RMID, NULL);
 
 	/* Now create pixman image */
-	output->hw_surface = pixman_image_create_bits(pixman_format, width, height, output->buf,
-		width * (bitsperpixel / 8));
+	output->renderbuffer =
+		renderer->pixman->create_image_from_ptr(&output->base,
+							pfmt, width, height,
+							output->buf,
+							width * (bitsperpixel / 8));
 
 	output->gc = xcb_generate_id(b->conn);
 	xcb_create_gc(b->conn, output->gc, output->window, 0, NULL);
@@ -803,28 +836,19 @@ static int
 x11_output_switch_mode(struct weston_output *base, struct weston_mode *mode)
 {
 	struct x11_backend *b;
-	struct x11_output *output;
+	struct x11_output *output = to_x11_output(base);
+	struct weston_size fb_size;
 	static uint32_t values[2];
-	int ret;
 
-	if (base == NULL) {
-		weston_log("output is NULL.\n");
-		return -1;
-	}
+	assert(output);
 
-	if (mode == NULL) {
-		weston_log("mode is NULL.\n");
-		return -1;
-	}
+	b = output->backend;
 
-        b = to_x11_backend(base->compositor);
-        output = to_x11_output(base);
-
-        if (mode->width == output->mode.width &&
+	if (mode->width == output->mode.width &&
 	    mode->height == output->mode.height)
-	        return 0;
+		return 0;
 
-        if (mode->width < WINDOW_MIN_WIDTH || mode->width > WINDOW_MAX_WIDTH)
+	if (mode->width < WINDOW_MIN_WIDTH || mode->width > WINDOW_MAX_WIDTH)
 		return -1;
 
 	if (mode->height < WINDOW_MIN_HEIGHT || mode->height > WINDOW_MAX_HEIGHT)
@@ -842,38 +866,22 @@ x11_output_switch_mode(struct weston_output *base, struct weston_mode *mode)
 				     XCB_CONFIG_WINDOW_HEIGHT, values);
 	}
 
-	output->mode.width = mode->width;
-	output->mode.height = mode->height;
+	fb_size.width = output->mode.width = mode->width;
+	fb_size.height = output->mode.height = mode->height;
 
-	if (b->use_pixman) {
-		pixman_renderer_output_destroy(&output->base);
+	weston_renderer_resize_output(&output->base, &fb_size, NULL);
+
+	if (base->compositor->renderer->type == WESTON_RENDERER_PIXMAN) {
+		const struct pixel_format_info *pfmt;
 		x11_output_deinit_shm(b, output);
-
-		if (x11_output_init_shm(b, output,
-		                        output->base.current_mode->width,
-		                        output->base.current_mode->height) < 0) {
+		pfmt = x11_output_get_shm_pixel_format(output);
+		if (!pfmt)
+			return -1;
+		if (x11_output_init_shm(b, output, pfmt,
+					fb_size.width, fb_size.height) < 0) {
 			weston_log("Failed to initialize SHM for the X11 output\n");
 			return -1;
 		}
-
-		if (pixman_renderer_output_create(&output->base,
-				PIXMAN_RENDERER_OUTPUT_USE_SHADOW) < 0) {
-			weston_log("Failed to create pixman renderer for output\n");
-			x11_output_deinit_shm(b, output);
-			return -1;
-		}
-	} else {
-		Window xid = (Window) output->window;
-
-		gl_renderer->output_destroy(&output->base);
-
-		ret = gl_renderer->output_window_create(&output->base,
-						        (EGLNativeWindowType) output->window,
-						        &xid,
-						        x11_formats,
-						        ARRAY_LENGTH(x11_formats));
-		if (ret < 0)
-			return -1;
 	}
 
 	output->resize_pending = false;
@@ -885,19 +893,24 @@ x11_output_switch_mode(struct weston_output *base, struct weston_mode *mode)
 static int
 x11_output_disable(struct weston_output *base)
 {
+	const struct weston_renderer *renderer = base->compositor->renderer;
 	struct x11_output *output = to_x11_output(base);
-	struct x11_backend *backend = to_x11_backend(base->compositor);
+	struct x11_backend *backend;
+
+	assert(output);
+
+	backend = output->backend;
 
 	if (!output->base.enabled)
 		return 0;
 
 	wl_event_source_remove(output->finish_frame_timer);
 
-	if (backend->use_pixman) {
-		pixman_renderer_output_destroy(&output->base);
+	if (renderer->type == WESTON_RENDERER_PIXMAN) {
 		x11_output_deinit_shm(backend, output);
+		renderer->pixman->output_destroy(&output->base);
 	} else {
-		gl_renderer->output_destroy(&output->base);
+		renderer->gl->output_destroy(&output->base);
 	}
 
 	xcb_destroy_window(backend->conn, output->window);
@@ -911,6 +924,8 @@ x11_output_destroy(struct weston_output *base)
 {
 	struct x11_output *output = to_x11_output(base);
 
+	assert(output);
+
 	x11_output_disable(&output->base);
 	weston_output_release(&output->base);
 
@@ -920,8 +935,14 @@ x11_output_destroy(struct weston_output *base)
 static int
 x11_output_enable(struct weston_output *base)
 {
+	const struct weston_renderer *renderer = base->compositor->renderer;
 	struct x11_output *output = to_x11_output(base);
-	struct x11_backend *b = to_x11_backend(base->compositor);
+	const struct weston_mode *mode = output->base.current_mode;
+	struct x11_backend *b;
+
+	assert(output);
+
+	b = output->backend;
 
 	static const char name[] = "Weston Compositor";
 	static const char class[] = "weston-1\0Weston Compositor";
@@ -960,8 +981,7 @@ x11_output_enable(struct weston_output *base)
 			  output->window,
 			  screen->root,
 			  0, 0,
-			  output->base.current_mode->width,
-			  output->base.current_mode->height,
+			  mode->width, mode->height,
 			  0,
 			  XCB_WINDOW_CLASS_INPUT_OUTPUT,
 			  screen->root_visual,
@@ -1021,17 +1041,25 @@ x11_output_enable(struct weston_output *base)
 	if (b->fullscreen)
 		x11_output_wait_for_map(b, output);
 
-	if (b->use_pixman) {
-		if (x11_output_init_shm(b, output,
-					output->base.current_mode->width,
-					output->base.current_mode->height) < 0) {
-			weston_log("Failed to initialize SHM for the X11 output\n");
+	if (renderer->type == WESTON_RENDERER_PIXMAN) {
+		const struct pixman_renderer_output_options options = {
+			.use_shadow = true,
+			.fb_size = {
+				.width = mode->width,
+				.height = mode->height
+			},
+			.format = x11_output_get_shm_pixel_format(output)
+		};
+		if (!options.format)
+			goto err;
+		if (renderer->pixman->output_create(&output->base, &options) < 0) {
+			weston_log("Failed to create pixman renderer for output\n");
 			goto err;
 		}
-		if (pixman_renderer_output_create(&output->base,
-				PIXMAN_RENDERER_OUTPUT_USE_SHADOW) < 0) {
-			weston_log("Failed to create pixman renderer for output\n");
-			x11_output_deinit_shm(b, output);
+		if (x11_output_init_shm(b, output, options.format,
+					mode->width, mode->height) < 0) {
+			weston_log("Failed to initialize SHM for the X11 output\n");
+			renderer->pixman->output_destroy(&output->base);
 			goto err;
 		}
 
@@ -1040,13 +1068,20 @@ x11_output_enable(struct weston_output *base)
 		/* eglCreatePlatformWindowSurfaceEXT takes a Window*
 		 * but eglCreateWindowSurface takes a Window. */
 		Window xid = (Window) output->window;
+		const struct gl_renderer_output_options options = {
+			.window_for_legacy = (EGLNativeWindowType) (uintptr_t) output->window,
+			.window_for_platform = &xid,
+			.formats = b->formats,
+			.formats_count = b->formats_count,
+			.area.x = 0,
+			.area.y = 0,
+			.area.width = mode->width,
+			.area.height = mode->height,
+			.fb_size.width = mode->width,
+			.fb_size.height = mode->height,
+		};
 
-		ret = gl_renderer->output_window_create(
-					&output->base,
-					(EGLNativeWindowType) output->window,
-					&xid,
-					x11_formats,
-					ARRAY_LENGTH(x11_formats));
+		ret = renderer->gl->output_window_create(base, &options);
 		if (ret < 0)
 			goto err;
 
@@ -1064,9 +1099,7 @@ x11_output_enable(struct weston_output *base)
 		wl_event_loop_add_timer(loop, finish_frame_handler, output);
 
 	weston_log("x11 output %dx%d, window id %d\n",
-		   output->base.current_mode->width,
-		   output->base.current_mode->height,
-		   output->window);
+		   mode->width, mode->height, output->window);
 
 	return 0;
 
@@ -1081,11 +1114,17 @@ static int
 x11_output_set_size(struct weston_output *base, int width, int height)
 {
 	struct x11_output *output = to_x11_output(base);
-	struct x11_backend *b = to_x11_backend(base->compositor);
+	struct x11_backend *b;
 	struct weston_head *head;
-	xcb_screen_t *scrn = b->screen;
+	xcb_screen_t *scrn;
 	int output_width, output_height;
 
+	if (!output)
+		return -1;
+
+	b = output->backend;
+	scrn = b->screen;
+
 	/* We can only be called once. */
 	assert(!output->base.current_mode);
 
@@ -1132,8 +1171,10 @@ x11_output_set_size(struct weston_output *base, int width, int height)
 }
 
 static struct weston_output *
-x11_output_create(struct weston_compositor *compositor, const char *name)
+x11_output_create(struct weston_backend *backend, const char *name)
 {
+	struct x11_backend *b = container_of(backend, struct x11_backend, base);
+	struct weston_compositor *compositor = b->compositor;
 	struct x11_output *output;
 
 	/* name can't be NULL. */
@@ -1150,14 +1191,17 @@ x11_output_create(struct weston_compositor *compositor, const char *name)
 	output->base.enable = x11_output_enable;
 	output->base.attach_head = NULL;
 
+	output->backend = b;
+
 	weston_compositor_add_pending_output(&output->base, compositor);
 
 	return &output->base;
 }
 
 static int
-x11_head_create(struct weston_compositor *compositor, const char *name)
+x11_head_create(struct weston_backend *base, const char *name)
 {
+	struct x11_backend *backend = to_x11_backend(base);
 	struct x11_head *head;
 
 	assert(name);
@@ -1167,15 +1211,22 @@ x11_head_create(struct weston_compositor *compositor, const char *name)
 		return -1;
 
 	weston_head_init(&head->base, name);
+
+	head->base.backend = &backend->base;
+
 	weston_head_set_connection_status(&head->base, true);
-	weston_compositor_add_head(compositor, &head->base);
+	weston_compositor_add_head(backend->compositor, &head->base);
 
 	return 0;
 }
 
 static void
-x11_head_destroy(struct x11_head *head)
+x11_head_destroy(struct weston_head *base)
 {
+	struct x11_head *head = to_x11_head(base);
+
+	assert(head);
+
 	weston_head_release(&head->base);
 	free(head);
 }
@@ -1377,7 +1428,7 @@ x11_backend_deliver_motion_event(struct x11_backend *b,
 				 xcb_generic_event_t *event)
 {
 	struct x11_output *output;
-	double x, y;
+	struct weston_coord_global pos;
 	struct weston_pointer_motion_event motion_event = { 0 };
 	xcb_motion_notify_event_t *motion_notify =
 			(xcb_motion_notify_event_t *) event;
@@ -1389,23 +1440,19 @@ x11_backend_deliver_motion_event(struct x11_backend *b,
 	if (!output)
 		return;
 
-	weston_output_transform_coordinate(&output->base,
-					   motion_notify->event_x,
-					   motion_notify->event_y,
-					   &x, &y);
+	pos = weston_coord_global_from_output_point(motion_notify->event_x,
+						    motion_notify->event_y,
+						    &output->base);
 
 	motion_event = (struct weston_pointer_motion_event) {
 		.mask = WESTON_POINTER_MOTION_REL,
-		.dx = x - b->prev_x,
-		.dy = y - b->prev_y
+		.rel = weston_coord_sub(pos.c, b->prev_pos.c),
 	};
-
 	weston_compositor_get_time(&time);
 	notify_motion(&b->core_seat, &time, &motion_event);
 	notify_pointer_frame(&b->core_seat);
 
-	b->prev_x = x;
-	b->prev_y = y;
+	b->prev_pos = pos;
 }
 
 static void
@@ -1413,7 +1460,7 @@ x11_backend_deliver_enter_event(struct x11_backend *b,
 				xcb_generic_event_t *event)
 {
 	struct x11_output *output;
-	double x, y;
+	struct weston_coord_global pos;
 
 	xcb_enter_notify_event_t *enter_notify =
 			(xcb_enter_notify_event_t *) event;
@@ -1425,14 +1472,13 @@ x11_backend_deliver_enter_event(struct x11_backend *b,
 	if (!output)
 		return;
 
-	weston_output_transform_coordinate(&output->base,
-					   enter_notify->event_x,
-					   enter_notify->event_y, &x, &y);
+	pos = weston_coord_global_from_output_point(enter_notify->event_x,
+						    enter_notify->event_y,
+						    &output->base);
 
-	notify_pointer_focus(&b->core_seat, &output->base, x, y);
+	notify_pointer_focus(&b->core_seat, &output->base, pos);
 
-	b->prev_x = x;
-	b->prev_y = y;
+	b->prev_pos = pos;
 }
 
 static int
@@ -1452,7 +1498,7 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 {
 	struct x11_backend *b = data;
 	struct x11_output *output;
-	xcb_generic_event_t *event, *prev;
+	xcb_generic_event_t *event;
 	xcb_client_message_event_t *client_message;
 	xcb_enter_notify_event_t *enter_notify;
 	xcb_key_press_event_t *key_press, *key_release;
@@ -1468,16 +1514,15 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 	int count;
 	struct timespec time;
 
-	prev = NULL;
 	count = 0;
 	while (x11_backend_next_event(b, &event, mask)) {
 		response_type = event->response_type & ~0x80;
 
-		switch (prev ? prev->response_type & ~0x80 : 0x80) {
+		switch (b->prev_event ? b->prev_event->response_type & ~0x80 : 0x80) {
 		case XCB_KEY_RELEASE:
 			/* Suppress key repeat events; this is only used if we
 			 * don't have XCB XKB support. */
-			key_release = (xcb_key_press_event_t *) prev;
+			key_release = (xcb_key_press_event_t *) b->prev_event;
 			key_press = (xcb_key_press_event_t *) event;
 			if (response_type == XCB_KEY_PRESS &&
 			    key_release->time == key_press->time &&
@@ -1485,8 +1530,8 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 				/* Don't deliver the held key release
 				 * event or the new key press event. */
 				free(event);
-				free(prev);
-				prev = NULL;
+				free(b->prev_event);
+				b->prev_event = NULL;
 				continue;
 			} else {
 				/* Deliver the held key release now
@@ -1499,8 +1544,8 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 					   key_release->detail - 8,
 					   WL_KEYBOARD_KEY_STATE_RELEASED,
 					   STATE_UPDATE_AUTOMATIC);
-				free(prev);
-				prev = NULL;
+				free(b->prev_event);
+				b->prev_event = NULL;
 				break;
 			}
 
@@ -1524,8 +1569,8 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 			notify_keyboard_focus_in(&b->core_seat, &b->keys,
 						 STATE_UPDATE_AUTOMATIC);
 
-			free(prev);
-			prev = NULL;
+			free(b->prev_event);
+			b->prev_event = NULL;
 			break;
 
 		default:
@@ -1550,7 +1595,7 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 			/* If we don't have XKB, we need to use the lame
 			 * autorepeat detection above. */
 			if (!b->has_xkb) {
-				prev = event;
+				b->prev_event = event;
 				break;
 			}
 			key_release = (xcb_key_press_event_t *) event;
@@ -1589,7 +1634,7 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 				break;
 			if (!b->has_xkb)
 				update_xkb_state_from_core(b, enter_notify->state);
-			notify_pointer_focus(&b->core_seat, NULL, 0, 0);
+			clear_pointer_focus(&b->core_seat);
 			break;
 
 		case XCB_CLIENT_MESSAGE:
@@ -1645,7 +1690,7 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 			if (focus_in->mode == XCB_NOTIFY_MODE_WHILE_GRABBED)
 				break;
 
-			prev = event;
+			b->prev_event = event;
 			break;
 
 		case XCB_FOCUS_OUT:
@@ -1679,13 +1724,13 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 #endif
 
 		count++;
-		if (prev != event)
-			free (event);
+		if (b->prev_event != event)
+			free(event);
 	}
 
-	switch (prev ? prev->response_type & ~0x80 : 0x80) {
+	switch (b->prev_event ? b->prev_event->response_type & ~0x80 : 0x80) {
 	case XCB_KEY_RELEASE:
-		key_release = (xcb_key_press_event_t *) prev;
+		key_release = (xcb_key_press_event_t *) b->prev_event;
 		update_xkb_state_from_core(b, key_release->state);
 		weston_compositor_get_time(&time);
 		notify_key(&b->core_seat,
@@ -1693,8 +1738,8 @@ x11_backend_handle_event(int fd, uint32_t mask, void *data)
 			   key_release->detail - 8,
 			   WL_KEYBOARD_KEY_STATE_RELEASED,
 			   STATE_UPDATE_AUTOMATIC);
-		free(prev);
-		prev = NULL;
+		free(b->prev_event);
+		b->prev_event = NULL;
 		break;
 	default:
 		break;
@@ -1784,42 +1829,27 @@ x11_backend_get_wm_info(struct x11_backend *c)
 }
 
 static void
-x11_destroy(struct weston_compositor *ec)
+x11_destroy(struct weston_backend *base)
 {
-	struct x11_backend *backend = to_x11_backend(ec);
-	struct weston_head *base, *next;
+	struct x11_backend *backend = container_of(base, struct x11_backend, base);
+	struct weston_compositor *ec = backend->compositor;
+	struct weston_head *head, *next;
 
 	wl_event_source_remove(backend->xcb_source);
 	x11_input_destroy(backend);
 
 	weston_compositor_shutdown(ec); /* destroys outputs, too */
 
-	wl_list_for_each_safe(base, next, &ec->head_list, compositor_link)
-		x11_head_destroy(to_x11_head(base));
+	wl_list_for_each_safe(head, next, &ec->head_list, compositor_link) {
+		if (to_x11_head(head))
+			x11_head_destroy(head);
+	}
 
 	XCloseDisplay(backend->dpy);
+	free(backend->formats);
 	free(backend);
 }
 
-static int
-init_gl_renderer(struct x11_backend *b)
-{
-	int ret;
-
-	gl_renderer = weston_load_module("gl-renderer.so",
-					 "gl_renderer_interface");
-	if (!gl_renderer)
-		return -1;
-
-	ret = gl_renderer->display_create(b->compositor, EGL_PLATFORM_X11_KHR,
-					  (void *) b->dpy,
-					  EGL_WINDOW_BIT,
-					  x11_formats,
-					  ARRAY_LENGTH(x11_formats));
-
-	return ret;
-}
-
 static const struct weston_windowed_output_api api = {
 	x11_output_set_size,
 	x11_head_create,
@@ -1868,17 +1898,30 @@ x11_backend_create(struct weston_compositor *compositor,
 		config->fullscreen = 0;
 	}
 
-	b->use_pixman = config->use_pixman;
-	if (b->use_pixman) {
-		if (pixman_renderer_init(compositor) < 0) {
+	b->formats_count = ARRAY_LENGTH(x11_formats);
+	b->formats = pixel_format_get_array(x11_formats, b->formats_count);
+
+	if (config->renderer == WESTON_RENDERER_PIXMAN) {
+		if (weston_compositor_init_renderer(compositor,
+						    WESTON_RENDERER_PIXMAN,
+						    NULL) < 0) {
 			weston_log("Failed to initialize pixman renderer for X11 backend\n");
 			goto err_xdisplay;
 		}
 	}
-	else if (init_gl_renderer(b) < 0) {
-		goto err_xdisplay;
+	else {
+		const struct gl_renderer_display_options options = {
+			.egl_platform = EGL_PLATFORM_X11_KHR,
+			.egl_native_display = b->dpy,
+			.egl_surface_type = EGL_WINDOW_BIT,
+			.formats = b->formats,
+			.formats_count = b->formats_count,
+		};
+		if (weston_compositor_init_renderer(compositor,
+						    WESTON_RENDERER_GL,
+						    &options.base) < 0)
+			goto err_xdisplay;
 	}
-	weston_log("Using %s renderer\n", config->use_pixman ? "pixman" : "gl");
 
 	b->base.destroy = x11_destroy;
 	b->base.create_output = x11_output_create;
@@ -1925,6 +1968,7 @@ err_renderer:
 err_xdisplay:
 	XCloseDisplay(b->dpy);
 err_free:
+	free(b->formats);
 	free(b);
 	return NULL;
 }
