@@ -238,8 +238,6 @@ weston_log_subscription_get_data(struct weston_log_subscription *sub)
  * subscription
  * @param scope the scope in order to add the subscription to the scope's
  * subscription list
- * @returns a weston_log_subscription object in case of success, or NULL
- * otherwise
  *
  * @sa weston_log_subscription_destroy, weston_log_subscription_remove,
  * weston_log_subscription_add
@@ -462,7 +460,7 @@ compositor_destroy_listener(struct wl_listener *listener, void *data)
 	struct weston_log_context *log_ctx =
 		wl_container_of(listener, log_ctx, compositor_destroy_listener);
 
-	/* We have to keep this list initalized as weston_log_ctx_destroy() has
+	/* We have to keep this list initialized as weston_log_ctx_destroy() has
 	 * to check if there's any compositor destroy listener registered */
 	wl_list_remove(&log_ctx->compositor_destroy_listener.link);
 	wl_list_init(&log_ctx->compositor_destroy_listener.link);
@@ -745,7 +743,7 @@ weston_log_subscription_complete(struct weston_log_subscription *sub)
  *
  * Complete the log scope, calling each stream's complete callback if one was
  * installed/created. This can be useful to signal the reading end that the
- * data has been transmited and should no longer expect that written over the
+ * data has been transmitted and should no longer expect that written over the
  * stream. Particularly useful for the weston-debug protocol.
  *
  * @memberof weston_log_scope
@@ -913,6 +911,56 @@ weston_log_scope_timestamp(struct weston_log_scope *scope,
 	return buf;
 }
 
+/** Returns a timestamp useful for adding it to a log scope.
+ *
+ * @example
+ * char timestr[128];
+ * static int cached_dm = -1;
+ * char *time_buff = weston_log_timestamp(timestr, sizeof(timestr),  &cached_dm);
+ * weston_log_scope_printf(log_scope, "%s %s", time_buff, other_data);
+ *
+ * @param buf a user-supplied buffer
+ * @param len user-supplied length of the buffer
+ * @param cached_tm_mday a cached day of the month, as an integer. Setting this
+ * pointer different from NULL, to an integer value other than was retrieved as
+ * current day of the month, would add an additional line under the form of
+ * 'Date: Y-m-d Z\n'. Setting the pointer to NULL would not print any date, nor
+ * if the value matches the current day of month. Helps identify logs that
+ * spawn multiple days, while still having a shorter time stamp format.
+ * @ingroup log
+ */
+WL_EXPORT char *
+weston_log_timestamp(char *buf, size_t len, int *cached_tm_mday)
+{
+       struct timeval tv;
+       struct tm *brokendown_time;
+       char datestr[128];
+       char timestr[128];
+
+       gettimeofday(&tv, NULL);
+
+       brokendown_time = localtime(&tv.tv_sec);
+       if (brokendown_time == NULL) {
+               snprintf(buf, len, "%s", "[(NULL)localtime] ");
+               return buf;
+       }
+
+       memset(datestr, 0, sizeof(datestr));
+       if (cached_tm_mday && brokendown_time->tm_mday != *cached_tm_mday) {
+               strftime(datestr, sizeof(datestr), "Date: %Y-%m-%d %Z\n",
+                        brokendown_time);
+               *cached_tm_mday = brokendown_time->tm_mday;
+       }
+
+       strftime(timestr, sizeof(timestr), "%H:%M:%S", brokendown_time);
+       /* if datestr is empty it prints only timestr*/
+       snprintf(buf, len, "%s[%s.%03li]", datestr,
+                timestr, (tv.tv_usec / 1000));
+
+       return buf;
+}
+
+
 void
 weston_log_subscriber_release(struct weston_log_subscriber *subscriber)
 {
