@@ -23,10 +23,13 @@
  * SOFTWARE.
  */
 
-#include "pipewire-plugin.h"
+#include "config.h"
+
+#include <libweston/pipewire-plugin.h>
 #include "backend.h"
 #include "libweston-internal.h"
 #include "shared/timespec-util.h"
+#include "shared/string-helpers.h"
 #include <libweston/backend-drm.h>
 #include <libweston/weston-log.h>
 
@@ -34,21 +37,17 @@
 #include <errno.h>
 #include <unistd.h>
 
+#include <pipewire/pipewire.h>
+
 #include <spa/param/format-utils.h>
 #include <spa/param/video/format-utils.h>
 #include <spa/utils/defs.h>
 
-#include <pipewire/pipewire.h>
+#include <spa/buffer/meta.h>
+#include <spa/utils/result.h>
 
 #define PROP_RANGE(min, max) 2, (min), (max)
 
-struct type {
-	struct spa_type_media_type media_type;
-	struct spa_type_media_subtype media_subtype;
-	struct spa_type_format_video format_video;
-	struct spa_type_video_format video_format;
-};
-
 struct weston_pipewire {
 	struct weston_compositor *compositor;
 	struct wl_list output_list;
@@ -60,17 +59,14 @@ struct weston_pipewire {
 	struct pw_loop *loop;
 	struct wl_event_source *loop_source;
 
+	struct pw_context *context;
 	struct pw_core *core;
 	struct pw_type *t;
-	struct type type;
-
-	struct pw_remote *remote;
-	struct spa_hook remote_listener;
+	struct spa_hook core_listener;
 };
 
 struct pipewire_output {
 	struct weston_output *output;
-	void (*saved_destroy)(struct weston_output *output);
 	int (*saved_enable)(struct weston_output *output);
 	int (*saved_disable)(struct weston_output *output);
 	int (*saved_start_repaint_loop)(struct weston_output *output);
@@ -100,14 +96,6 @@ struct pipewire_frame_data {
 	struct wl_event_source *fence_sync_event_source;
 };
 
-static inline void init_type(struct type *type, struct spa_type_map *map)
-{
-	spa_type_media_type_map(map, &type->media_type);
-	spa_type_media_subtype_map(map, &type->media_subtype);
-	spa_type_format_video_map(map, &type->format_video);
-	spa_type_video_format_map(map, &type->video_format);
-}
-
 static void
 pipewire_debug_impl(struct weston_pipewire *pipewire,
 		    struct pipewire_output *output,
@@ -141,16 +129,6 @@ pipewire_debug_impl(struct weston_pipewire *pipewire,
 	free(logstr);
 }
 
-static void
-pipewire_debug(struct weston_pipewire *pipewire, const char *fmt, ...)
-{
-	va_list ap;
-
-	va_start(ap, fmt);
-	pipewire_debug_impl(pipewire, NULL, fmt, ap);
-	va_end(ap);
-}
-
 static void
 pipewire_output_debug(struct pipewire_output *output, const char *fmt, ...)
 {
@@ -171,6 +149,16 @@ lookup_pipewire_output(struct weston_output *base_output)
 	struct weston_pipewire *pipewire = weston_pipewire_get(c);
 	struct pipewire_output *output;
 
+	/* XXX: This could happen on the compositor shutdown path with our
+	 * destroy listener being removed, and pipewire_output_destroy() being
+	 * called as a virtual destructor.
+	 *
+	 * See https://gitlab.freedesktop.org/wayland/weston/-/issues/591 for
+	 * an alternative to the shutdown sequence.
+	 */
+	if (!pipewire)
+		return NULL;
+
 	wl_list_for_each(output, &pipewire->output_list, link) {
 		if (output->output == base_output)
 			return output;
@@ -185,7 +173,6 @@ pipewire_output_handle_frame(struct pipewire_output *output, int fd,
 	const struct weston_drm_virtual_output_api *api =
 		output->pipewire->virtual_output_api;
 	size_t size = output->output->height * stride;
-	struct pw_type *t = output->pipewire->t;
 	struct pw_buffer *buffer;
 	struct spa_buffer *spa_buffer;
 	struct spa_meta_header *h;
@@ -203,8 +190,12 @@ pipewire_output_handle_frame(struct pipewire_output *output, int fd,
 
 	spa_buffer = buffer->buffer;
 
-	if ((h = spa_buffer_find_meta(spa_buffer, t->meta.Header))) {
-		h->pts = -1;
+	if ((h = spa_buffer_find_meta_data(spa_buffer, SPA_META_Header,
+				     sizeof(struct spa_meta_header)))) {
+		struct timespec ts;
+
+		clock_gettime(CLOCK_MONOTONIC, &ts);
+		h->pts = SPA_TIMESPEC_TO_NSEC(&ts);
 		h->flags = 0;
 		h->seq = output->seq++;
 		h->dts_offset = 0;
@@ -332,17 +323,19 @@ pipewire_output_destroy(struct weston_output *base_output)
 	struct pipewire_output *output = lookup_pipewire_output(base_output);
 	struct weston_mode *mode, *next;
 
+	if (!output)
+		return;
+
+	weston_head_release(output->head);
+
 	wl_list_for_each_safe(mode, next, &base_output->mode_list, link) {
 		wl_list_remove(&mode->link);
 		free(mode);
 	}
 
-	output->saved_destroy(base_output);
-
 	pw_stream_destroy(output->stream);
 
 	wl_list_remove(&output->link);
-	weston_head_release(output->head);
 	free(output->head);
 	free(output);
 }
@@ -375,34 +368,28 @@ pipewire_set_dpms(struct weston_output *base_output, enum dpms_enum level)
 static int
 pipewire_output_connect(struct pipewire_output *output)
 {
-	struct weston_pipewire *pipewire = output->pipewire;
-	struct type *type = &pipewire->type;
 	uint8_t buffer[1024];
 	struct spa_pod_builder builder =
 		SPA_POD_BUILDER_INIT(buffer, sizeof(buffer));
 	const struct spa_pod *params[1];
-	struct pw_type *t = pipewire->t;
 	int frame_rate = output->output->current_mode->refresh / 1000;
 	int width = output->output->width;
 	int height = output->output->height;
 	int ret;
 
-	params[0] = spa_pod_builder_object(&builder,
-		t->param.idEnumFormat, t->spa_format,
-		"I", type->media_type.video,
-		"I", type->media_subtype.raw,
-		":", type->format_video.format,
-		"I", type->video_format.BGRx,
-		":", type->format_video.size,
-		"R", &SPA_RECTANGLE(width, height),
-		":", type->format_video.framerate,
-		"F", &SPA_FRACTION(0, 1),
-		":", type->format_video.max_framerate,
-		"Fru", &SPA_FRACTION(frame_rate, 1),
-		       PROP_RANGE(&SPA_FRACTION(1, 1),
-				  &SPA_FRACTION(frame_rate, 1)));
+	params[0] = spa_pod_builder_add_object(&builder,
+		SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat,
+		SPA_FORMAT_mediaType, SPA_POD_Id(SPA_MEDIA_TYPE_video),
+		SPA_FORMAT_mediaSubtype, SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
+		SPA_FORMAT_VIDEO_format, SPA_POD_Id(SPA_VIDEO_FORMAT_BGRx),
+		SPA_FORMAT_VIDEO_size, SPA_POD_Rectangle(&SPA_RECTANGLE(width, height)),
+		SPA_FORMAT_VIDEO_framerate, SPA_POD_Fraction(&SPA_FRACTION (0, 1)),
+		SPA_FORMAT_VIDEO_maxFramerate,
+		SPA_POD_CHOICE_RANGE_Fraction(&SPA_FRACTION(frame_rate, 1),
+			&SPA_FRACTION(1, 1),
+			&SPA_FRACTION(frame_rate, 1)));
 
-	ret = pw_stream_connect(output->stream, PW_DIRECTION_OUTPUT, NULL,
+	ret = pw_stream_connect(output->stream, PW_DIRECTION_OUTPUT, SPA_ID_INVALID,
 				(PW_STREAM_FLAG_DRIVER |
 				 PW_STREAM_FLAG_MAP_BUFFERS),
 				params, 1);
@@ -482,26 +469,23 @@ pipewire_output_stream_state_changed(void *data, enum pw_stream_state old,
 }
 
 static void
-pipewire_output_stream_format_changed(void *data, const struct spa_pod *format)
+pipewire_output_stream_param_changed(void *data, uint32_t id, const struct spa_pod *format)
 {
 	struct pipewire_output *output = data;
-	struct weston_pipewire *pipewire = output->pipewire;
 	uint8_t buffer[1024];
 	struct spa_pod_builder builder =
 		SPA_POD_BUILDER_INIT(buffer, sizeof(buffer));
 	const struct spa_pod *params[2];
-	struct pw_type *t = pipewire->t;
 	int32_t width, height, stride, size;
 	const int bpp = 4;
 
 	if (!format) {
 		pipewire_output_debug(output, "format = None");
-		pw_stream_finish_format(output->stream, 0, NULL, 0);
+		pw_stream_update_params(output->stream, NULL, 0);
 		return;
 	}
 
-	spa_format_video_raw_parse(format, &output->video_format,
-				   &pipewire->type.format_video);
+	spa_format_video_raw_parse(format, &output->video_format);
 
 	width = output->video_format.size.width;
 	height = output->video_format.size.height;
@@ -510,29 +494,25 @@ pipewire_output_stream_format_changed(void *data, const struct spa_pod *format)
 
 	pipewire_output_debug(output, "format = %dx%d", width, height);
 
-	params[0] = spa_pod_builder_object(&builder,
-		t->param.idBuffers, t->param_buffers.Buffers,
-		":", t->param_buffers.size,
-		"i", size,
-		":", t->param_buffers.stride,
-		"i", stride,
-		":", t->param_buffers.buffers,
-		"iru", 4, PROP_RANGE(2, 8),
-		":", t->param_buffers.align,
-		"i", 16);
+	params[0] = spa_pod_builder_add_object(&builder,
+		SPA_TYPE_OBJECT_ParamBuffers, SPA_PARAM_Buffers,
+		SPA_PARAM_BUFFERS_size, SPA_POD_Int(size),
+		SPA_PARAM_BUFFERS_stride, SPA_POD_Int(stride),
+		SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(4, 2, 8),
+		SPA_PARAM_BUFFERS_align, SPA_POD_Int(16));
 
-	params[1] = spa_pod_builder_object(&builder,
-		t->param.idMeta, t->param_meta.Meta,
-		":", t->param_meta.type, "I", t->meta.Header,
-		":", t->param_meta.size, "i", sizeof(struct spa_meta_header));
+	params[1] = spa_pod_builder_add_object(&builder,
+		SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
+		SPA_PARAM_META_type, SPA_POD_Id(SPA_META_Header),
+		SPA_PARAM_META_size, SPA_POD_Int(sizeof(struct spa_meta_header)));
 
-	pw_stream_finish_format(output->stream, 0, params, 2);
+	pw_stream_update_params(output->stream, params, 2);
 }
 
 static const struct pw_stream_events stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.state_changed = pipewire_output_stream_state_changed,
-	.format_changed = pipewire_output_stream_format_changed,
+	.param_changed = pipewire_output_stream_param_changed,
 };
 
 static struct weston_output *
@@ -546,6 +526,7 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	const char *model = "Virtual Display";
 	const char *serial_number = "unknown";
 	const char *connector_name = "pipewire";
+	char *remoting_name;
 
 	if (!name || !strlen(name))
 		return NULL;
@@ -560,7 +541,7 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	if (!head)
 		goto err;
 
-	output->stream = pw_stream_new(pipewire->remote, name, NULL);
+	output->stream = pw_stream_new(pipewire->core, name, NULL);
 	if (!output->stream) {
 		weston_log("Cannot initialize pipewire stream\n");
 		goto err;
@@ -569,14 +550,12 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	pw_stream_add_listener(output->stream, &output->stream_listener,
 			       &stream_events, output);
 
-	output->output = api->create_output(c, name);
+	output->output = api->create_output(c, name, pipewire_output_destroy);
 	if (!output->output) {
 		weston_log("Cannot create virtual output\n");
 		goto err;
 	}
 
-	output->saved_destroy = output->output->destroy;
-	output->output->destroy = pipewire_output_destroy;
 	output->saved_enable = output->output->enable;
 	output->output->enable = pipewire_output_enable;
 	output->saved_disable = output->output->disable;
@@ -584,7 +563,8 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 	output->pipewire = pipewire;
 	wl_list_insert(pipewire->output_list.prev, &output->link);
 
-	weston_head_init(head, connector_name);
+	str_printf(&remoting_name, "%s-%s", connector_name, name);
+	weston_head_init(head, remoting_name);
 	weston_head_set_subpixel(head, WL_OUTPUT_SUBPIXEL_NONE);
 	weston_head_set_monitor_strings(head, make, model, serial_number);
 	head->compositor = c;
@@ -592,6 +572,7 @@ pipewire_output_create(struct weston_compositor *c, char *name)
 
 	weston_output_attach_head(output->output, head);
 
+	free(remoting_name);
 	pipewire_output_debug(output, "created");
 
 	return output->output;
@@ -666,13 +647,19 @@ weston_pipewire_destroy(struct wl_listener *l, void *data)
 {
 	struct weston_pipewire *pipewire =
 		wl_container_of(l, pipewire, destroy_listener);
+	struct pipewire_output *p_output, *p_output_next;
 
 	weston_log_scope_destroy(pipewire->debug);
 	pipewire->debug = NULL;
 
+	wl_list_for_each_safe(p_output, p_output_next, &pipewire->output_list, link)
+		pipewire_output_destroy(p_output->output);
+
 	wl_event_source_remove(pipewire->loop_source);
 	pw_loop_leave(pipewire->loop);
 	pw_loop_destroy(pipewire->loop);
+
+	free(pipewire);
 }
 
 static struct weston_pipewire *
@@ -705,31 +692,15 @@ weston_pipewire_loop_handler(int fd, uint32_t mask, void *data)
 }
 
 static void
-weston_pipewire_state_changed(void *data, enum pw_remote_state old,
-			      enum pw_remote_state state, const char *error)
+weston_pipewire_error(void *data, uint32_t id, int seq, int res,
+			      const char *error)
 {
-	struct weston_pipewire *pipewire = data;
-
-	pipewire_debug(pipewire, "[remote] state changed %s -> %s",
-		       pw_remote_state_as_string(old),
-		       pw_remote_state_as_string(state));
-
-	switch (state) {
-	case PW_REMOTE_STATE_ERROR:
-		weston_log("pipewire remote error: %s\n", error);
-		break;
-	case PW_REMOTE_STATE_CONNECTED:
-		weston_log("connected to pipewire daemon\n");
-		break;
-	default:
-		break;
-	}
+	weston_log("pipewire remote error: %s\n", error);
 }
 
-
-static const struct pw_remote_events remote_events = {
-	PW_VERSION_REMOTE_EVENTS,
-	.state_changed = weston_pipewire_state_changed,
+static const struct pw_core_events core_events = {
+	PW_VERSION_CORE_EVENTS,
+	.error = weston_pipewire_error,
 };
 
 static int
@@ -745,38 +716,12 @@ weston_pipewire_init(struct weston_pipewire *pipewire)
 
 	pw_loop_enter(pipewire->loop);
 
-	pipewire->core = pw_core_new(pipewire->loop, NULL);
-	pipewire->t = pw_core_get_type(pipewire->core);
-	init_type(&pipewire->type, pipewire->t->map);
+	pipewire->context = pw_context_new(pipewire->loop, NULL, 0);
+	pipewire->core = pw_context_connect(pipewire->context, NULL, 0);
 
-	pipewire->remote = pw_remote_new(pipewire->core, NULL, 0);
-	pw_remote_add_listener(pipewire->remote,
-			       &pipewire->remote_listener,
-			       &remote_events, pipewire);
-
-	pw_remote_connect(pipewire->remote);
-
-	while (true) {
-		enum pw_remote_state state;
-		const char *error = NULL;
-		int ret;
-
-		state = pw_remote_get_state(pipewire->remote, &error);
-		if (state == PW_REMOTE_STATE_CONNECTED)
-			break;
-
-		if (state == PW_REMOTE_STATE_ERROR) {
-			weston_log("pipewire error: %s\n", error);
-			goto err;
-		}
-
-		ret = pw_loop_iterate(pipewire->loop, -1);
-		if (ret < 0) {
-			weston_log("pipewire_loop_iterate failed: %s",
-				   spa_strerror(ret));
-			goto err;
-		}
-	}
+	pw_core_add_listener(pipewire->core,
+			       &pipewire->core_listener,
+			       &core_events, pipewire);
 
 	loop = wl_display_get_event_loop(pipewire->compositor->wl_display);
 	pipewire->loop_source =
@@ -786,12 +731,6 @@ weston_pipewire_init(struct weston_pipewire *pipewire)
 				     pipewire);
 
 	return 0;
-err:
-	if (pipewire->remote)
-		pw_remote_destroy(pipewire->remote);
-	pw_loop_leave(pipewire->loop);
-	pw_loop_destroy(pipewire->loop);
-	return -1;
 }
 
 static const struct weston_pipewire_api pipewire_api = {
