@@ -33,9 +33,7 @@
 #include <libweston/libweston.h>
 #include <libweston/xwayland-api.h>
 #include <libweston/weston-log.h>
-
-#define SEND_EVENT_MASK (0x80)
-#define EVENT_TYPE(event) ((event)->response_type & ~SEND_EVENT_MASK)
+#include "shared/xcb-xwayland.h"
 
 struct weston_xserver {
 	struct wl_display *wl_display;
@@ -63,6 +61,7 @@ struct weston_wm {
 	xcb_screen_t *screen;
 	struct hash_table *window_hash;
 	struct weston_xserver *server;
+	struct wl_global *xwayland_shell_global;
 	xcb_window_t wm_window;
 	struct weston_wm_window *focus_window;
 	struct theme *theme;
@@ -90,84 +89,22 @@ struct weston_wm {
 	int selection_property_set;
 	int flush_property_on_delete;
 	struct wl_listener selection_listener;
+	struct wl_listener seat_create_listener;
+	struct wl_listener seat_destroy_listener;
 
 	xcb_window_t dnd_window;
 	xcb_window_t dnd_owner;
 
-	struct {
-		xcb_atom_t		 wm_protocols;
-		xcb_atom_t		 wm_normal_hints;
-		xcb_atom_t		 wm_take_focus;
-		xcb_atom_t		 wm_delete_window;
-		xcb_atom_t		 wm_state;
-		xcb_atom_t		 wm_s0;
-		xcb_atom_t		 wm_client_machine;
-		xcb_atom_t		 net_wm_cm_s0;
-		xcb_atom_t		 net_wm_name;
-		xcb_atom_t		 net_wm_pid;
-		xcb_atom_t		 net_wm_icon;
-		xcb_atom_t		 net_wm_state;
-		xcb_atom_t		 net_wm_state_maximized_vert;
-		xcb_atom_t		 net_wm_state_maximized_horz;
-		xcb_atom_t		 net_wm_state_fullscreen;
-		xcb_atom_t		 net_wm_user_time;
-		xcb_atom_t		 net_wm_icon_name;
-		xcb_atom_t		 net_wm_desktop;
-		xcb_atom_t		 net_wm_window_type;
-		xcb_atom_t		 net_wm_window_type_desktop;
-		xcb_atom_t		 net_wm_window_type_dock;
-		xcb_atom_t		 net_wm_window_type_toolbar;
-		xcb_atom_t		 net_wm_window_type_menu;
-		xcb_atom_t		 net_wm_window_type_utility;
-		xcb_atom_t		 net_wm_window_type_splash;
-		xcb_atom_t		 net_wm_window_type_dialog;
-		xcb_atom_t		 net_wm_window_type_dropdown;
-		xcb_atom_t		 net_wm_window_type_popup;
-		xcb_atom_t		 net_wm_window_type_tooltip;
-		xcb_atom_t		 net_wm_window_type_notification;
-		xcb_atom_t		 net_wm_window_type_combo;
-		xcb_atom_t		 net_wm_window_type_dnd;
-		xcb_atom_t		 net_wm_window_type_normal;
-		xcb_atom_t		 net_wm_moveresize;
-		xcb_atom_t		 net_supporting_wm_check;
-		xcb_atom_t		 net_supported;
-		xcb_atom_t		 net_active_window;
-		xcb_atom_t		 motif_wm_hints;
-		xcb_atom_t		 clipboard;
-		xcb_atom_t		 clipboard_manager;
-		xcb_atom_t		 targets;
-		xcb_atom_t		 utf8_string;
-		xcb_atom_t		 wl_selection;
-		xcb_atom_t		 incr;
-		xcb_atom_t		 timestamp;
-		xcb_atom_t		 multiple;
-		xcb_atom_t		 compound_text;
-		xcb_atom_t		 text;
-		xcb_atom_t		 string;
-		xcb_atom_t		 window;
-		xcb_atom_t		 text_plain_utf8;
-		xcb_atom_t		 text_plain;
-		xcb_atom_t		 xdnd_selection;
-		xcb_atom_t		 xdnd_aware;
-		xcb_atom_t		 xdnd_enter;
-		xcb_atom_t		 xdnd_leave;
-		xcb_atom_t		 xdnd_drop;
-		xcb_atom_t		 xdnd_status;
-		xcb_atom_t		 xdnd_finished;
-		xcb_atom_t		 xdnd_type_list;
-		xcb_atom_t		 xdnd_action_copy;
-		xcb_atom_t		 wl_surface_id;
-		xcb_atom_t		 allow_commits;
-	} atom;
+	struct wl_list unpaired_surface_list;
+	bool shell_bound;
+
+	struct atom_x11 atom;
 };
 
 void
 dump_property(FILE *fp, struct weston_wm *wm, xcb_atom_t property,
 	      xcb_get_property_reply_t *reply);
 
-const char *
-get_atom_name(xcb_connection_t *c, xcb_atom_t atom);
-
 void
 weston_wm_selection_init(struct weston_wm *wm);
 int
